@@ -36,7 +36,8 @@ struct XRefEntry {
   int gen;
   int num;
   XRefEntryType type;
-  Object obj;
+  bool updated;
+  Object obj; //if this entry was updated, obj will contains the updated object
 };
 
 class XRef {
@@ -109,8 +110,9 @@ public:
 
   // Write access
   void setModifiedObject(Object* o, Ref r);
+  Ref addIndirectObject (Object* o);
   void add(int num, int gen,  Guint offs, GBool used);
-  void writeToFile(FILE* f);
+  void writeToFile(OutStream* outStr);
 
 private:
 
