@@ -59,6 +59,7 @@ static GooString* basename(GooString* str){
   return new GooString(str);
 }
 
+#if 0
 static GooString* Dirname(GooString* str){
   
   char *p=str->getCString();
@@ -68,6 +69,7 @@ static GooString* Dirname(GooString* str){
       return new GooString(p,i+1);
   return new GooString();
 } 
+#endif
 
 //------------------------------------------------------------------------
 // HtmlString
@@ -818,8 +820,8 @@ void HtmlOutputDev::doFrame(int firstPage){
   fName->append(".html");
 
   if (!(fContentsFrame = fopen(fName->getCString(), "w"))){
-    delete fName;
     error(-1, "Couldn't open html file '%s'", fName->getCString());
+    delete fName;
     return;
   }
   
@@ -932,8 +934,8 @@ HtmlOutputDev::HtmlOutputDev(char *fileName, char *title,
       if (!xml) right->append(".html");
       if (xml) right->append(".xml");
       if (!(page=fopen(right->getCString(),"w"))){
-	delete right;
 	error(-1, "Couldn't open html file '%s'", right->getCString());
+	delete right;
 	return;
       }  
       delete right;
@@ -994,10 +996,9 @@ HtmlOutputDev::~HtmlOutputDev() {
       delete pages;
 }
 
-
-
 void HtmlOutputDev::startPage(int pageNum, GfxState *state) {
-  /*if (mode&&!xml){
+#if 0
+  if (mode&&!xml){
     if (write){
       write=gFalse;
       GooString* fname=Dirname(Docname);
@@ -1012,7 +1013,8 @@ void HtmlOutputDev::startPage(int pageNum, GfxState *state) {
     // else 
       fprintf(tin,"ROTATE=%d neg %d neg translate\n",state->getX1(),state->getY1());  
     }
-  }*/
+  }
+#endif
 
   this->pageNum = pageNum;
   GooString *str=basename(Docname);
@@ -1084,8 +1086,6 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 			      int width, int height, GBool invert,
 			      GBool inlineImg) {
 
-  int i, j;
-  
   if (ignore||complexMode) {
     OutputDev::drawImageMask(state, ref, str, width, height, invert, inlineImg);
     return;
@@ -1098,14 +1098,6 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   int w0, h0, w1, h1;		// size of image
   double xt, yt, wt, ht;
   GBool rotate, xFlip, yFlip;
-  GBool dither;
-  int x, y;
-  int ix, iy;
-  int px1, px2, qx, dx;
-  int py1, py2, qy, dy;
-  Gulong pixel;
-  int nComps, nVals, nBits;
-  double r1, g1, b1;
  
   // get image position and size
   state->transform(0, 0, &xt, &yt);
@@ -1175,8 +1167,6 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 			  int width, int height, GfxImageColorMap *colorMap,
 			  int *maskColors, GBool inlineImg) {
 
-  int i, j;
-   
   if (ignore||complexMode) {
     OutputDev::drawImage(state, ref, str, width, height, colorMap, 
 			 maskColors, inlineImg);
@@ -1184,23 +1174,12 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   }
 
   FILE *f1;
-  ImageStream *imgStr;
-  Guchar pixBuf[4];
-  GfxColor color;
   int c;
   
   int x0, y0;			// top left corner of image
   int w0, h0, w1, h1;		// size of image
   double xt, yt, wt, ht;
   GBool rotate, xFlip, yFlip;
-  GBool dither;
-  int x, y;
-  int ix, iy;
-  int px1, px2, qx, dx;
-  int py1, py2, qy, dy;
-  Gulong pixel;
-  int nComps, nVals, nBits;
-  double r1, g1, b1;
  
   // get image position and size
   state->transform(0, 0, &xt, &yt);
